from abc import abstractmethod

import pandas_ta as ta

from Hmile.DataProvider import DataProvider

class DataTransformer:
    """Abstraction class to apply data transformation
    """

    def __init__(self, dataprovider : DataProvider) -> None:
        self.dataprovider = dataprovider

    @abstractmethod
    def transform(self):
        """Apply transformation
        """
        raise NotImplementedError()


class TaFeaturesTransformer(DataTransformer):
    def __init__(self, dataprovider : DataProvider) -> None:
        """Apply all technical analysis features to the data

        Args:
            dataprovider (Hmile.DataProvider.Dataprovider): Dataprovider to transform
            ta_features (list): list of technical analysis features to apply
        """
        super().__init__(dataprovider)

    def transform(self):
        data = self.dataprovider.getData()
        data.ta.strategy("all")
        return data



# TODO:
# - tafeatures
# - encoder


import torch

from Hmile.DataProvider import DataProvider

null = None

class DataTensorer:
    """Can be used to transform data into pytorch tensor
    """
    data = null
    norm_data = null
    min = null
    max = null
    size = null
    MIN = -10
    MAX = 10
    def __init__(
        self,
        dataprovider : DataProvider,
        nb_env,
        nb_data_per_session,
        device,
        mean_window_size=300,
        window_size=-1
        ) -> None:
        """Initialize the DataTensorer

        Args:
            dataprovider (DataProvider): Dataprovider to use to get the data
            nb_env (int): size of dim 3 of the tensor
            nb_data_per_session (str): size of dim 2 of the tensor
            device (str): pytorch device to use
            mean_window_size (int, optional): Window for rolling mean normalization. Defaults to 300.
            window_size (int, optional): size of dim 1. Defaults to -1.
        """        

        self.window_size = window_size
        self.mean_window_size = mean_window_size
        if DataTensorer.data == null :
            df = dataprovider.getData()
            df.fillna(method="ffill",inplace=True)
            df.fillna(method="bfill",inplace=True)
            DataTensorer.size = df.shape[1]
            DataTensorer.data = torch.tensor(df.to_numpy(), device=device)
            DataTensorer.norm_data = self.normalize(DataTensorer.data)
        
        self.size = DataTensorer.norm_data.shape[0]
        self.nb_env = nb_env
        self.nb_data_per_session = nb_data_per_session
        self.device = device
        self.create_tensor()
        

    def normalize(self, data):
        """Apply rolling mean and variance normalization. 
        The result is clipped between -10 and 10 
        
        Args:
            data (torch.tensor): tensor to normalize
        """

        mean = torch.clone(data) # moyenne glissante
        var = torch.clone(data) # variance glissante

        for i in range (data.size()[0]):
            if i < self.mean_window_size:
                mean[i] = torch.mean(data[0:self.mean_window_size], dim=0)
                var[i] = torch.var(data[0:self.mean_window_size], dim=0)
            else:
                mean[i] = torch.mean(data[i-self.mean_window_size:i], dim=0)
                var[i] = torch.var(data[i-self.mean_window_size:i], dim=0)

        diff = torch.sub(data, mean)
        out = torch.div(diff, torch.sqrt(var))
        return torch.clip(out, min=DataTensorer.MIN, max=DataTensorer.MAX)

    def create_tensor(self):
        self.current_step = torch.zeros(self.nb_env,device=self.device,dtype=torch.long)
        self.provider = torch.zeros(self.nb_env,device=self.device,dtype=torch.float64)
        self.reset()

    def reset_by_id(self,indices) :
        """Reset the current step for the given indices

        Args:
            indices (list): list of indices to reset
        """
        if indices.shape[0] == 0 :
            return True
        begin_indices = torch.randint(self.mean_window_size,self.size-self.nb_data_per_session-5,(indices.shape[0],),device=self.device,dtype=torch.long)
        self.current_step[indices] = begin_indices

    def reset(self):
        """Reset the current step for all indices
        """
        self.reset_by_id(torch.ones(self.nb_env,device=self.device,dtype=torch.bool))
                
    def get_indicators(self):
        """Returns normalized and unnormalized data
        
        Returns:
            tuple: (unnormalized data : torch.Tensor, normalized data : torch.Tensor)"""

        norm_indicators = DataTensorer.norm_data[self.current_step]
        indicators = DataTensorer.data[self.current_step]
        self.current_step+=1
        return norm_indicators, indicators

    def get_min_indices(self) -> list :
        """Returns the min for each feature

        Returns:
            list: result
        """
        return [-10 for _ in range(DataTensorer.size)] 

    def get_max_indices(self) -> list :
        """Returns the max for each feature

         Returns:
            list: result
        """
        return [10 for _ in range(DataTensorer.size)]


from locale import normalize
from logging.handlers import DatagramHandler
import pandas as pd
import yfinance as yf
import time

from random import randint
from datetime import datetime
from elasticsearch import Elasticsearch
import pickle
from datetime import timedelta

import numpy as np
import torch

from abc import ABC, abstractmethod

from Hmile.Exception import DataframeFormatException, DataProviderArgumentException


yahoointervalconverter = {
    'minute': '1m',
    'hour': '1h',
    'day': '1d'
}

# TODO: add format checker for pair, interval, start, end

class DataProvider(ABC):
    """
    Provide an abstraction layer on the way to get data from a source
    """
    def __init__(self, pair, interval, start, end) -> None:
        """Inialize a DataProvider

        Args:
            pair (str): the pair to get ex : BTCUSD
            interval (str): should be like day, hour or minute
            start (str): should be like 2020-12-31
            end (str): should be > start

        Raises:
            DataframeFormatException: When the dataframe does not correspond to Hmile norm
            DataProviderArgumentException: When the argument are not correct
        """
        self.checkArguments(pair, interval, start, end)
        self.pair = pair
        self.interval = interval
        self.start_date = start
        self.end_date = end

    CHECK_INDEX_MONOTONIC_INCREASING = True
    @abstractmethod
    def getData(self) -> pd.DataFrame:
        """
        Return a pandas dataframe with the data.
        The main columns are named be open, high, low, close, volume. In index is the date.
        The index name should be 'date'
        """
        raise NotImplementedError()
    
    def checkDataframe(self, dataframe):
        """Check if first columns in the dataframes are open, high, low, close, volume. 
        Check if index is a date and if the interval is the same between all rows"""
        columns = dataframe.columns
        if not columns[0] == 'open' or not columns[1] == 'high' or not columns[2] == 'low' or not columns[3] == 'close' or not columns[4] == 'volume':
            raise DataframeFormatException('The first columns in the dataframe should be open, high, low, close, volume', dataframe)
        if not isinstance(dataframe.index, pd.DatetimeIndex):
            raise DataframeFormatException('The index of the dataframe should be a date', dataframe)
        if not dataframe.index.is_monotonic_increasing:
            raise DataframeFormatException('The index of the dataframe should be monotonic increasing', dataframe)
        if not dataframe.index.is_unique:
            raise DataframeFormatException('The index of the dataframe should be unique', dataframe)
        if DataProvider.CHECK_INDEX_MONOTONIC_INCREASING and not dataframe.index.inferred_freq:
            raise Exception('The index of the dataframe should have a fixed interval', dataframe)
        if dataframe.index.name != 'date':
            raise DataframeFormatException('The index name should be date', dataframe)
    
    def normalizeColumnsOrder(self, dataframe):
        """Normalize the order of the columns to open, high, low, close, volume
        
        Args:
            dataframe (pd.DataFrame): The dataframe to treat


        Returns:
            pd.DataFrame: After traitement
        """
        ohlcv = ['open', 'high', 'low', 'close', 'volume']
        col_list = ohlcv + [col for col in dataframe.columns if col not in ohlcv]
        return dataframe.reindex(columns=col_list)
    

    def checkArguments(self, pair, interval, start, end):
        """Check if the arguments are valid. pair should be like BTCUSD, interval should be in yahoointervalconverter, start and end should be like YYYY-MM-DD
        start should be before end
        
        Args:
            pair (str): The pair to get
            interval (str): The interval of the data
            start (str): The start date
            end (str) The end date
        """
        if not pair:
            raise DataProviderArgumentException('pair should not be empty')
        if not interval:
            raise DataProviderArgumentException('interval should not be empty')
        if interval not in yahoointervalconverter.keys():
            print(interval)
            raise DataProviderArgumentException('interval should be in day, hour or minute')
        if not start:
            raise DataProviderArgumentException('start should not be empty')
        if not end:
            raise DataProviderArgumentException('end should not be empty')
        if not interval in yahoointervalconverter:
            raise DataProviderArgumentException('interval should be in ' + str(yahoointervalconverter))
        try:
            datetime.strptime(start, '%Y-%m-%d')
        except ValueError:
            raise Exception('start should be like YYYY-MM-DD')
        try:
            datetime.strptime(end, '%Y-%m-%d')
        except ValueError:
            raise Exception('end should be like YYYY-MM-DD')
        if datetime.strptime(start, '%Y-%m-%d') > datetime.strptime(end, '%Y-%m-%d'):
            raise Exception('start should be before end')

    
class YahooDataProvider(DataProvider):
    """
    Get data from Yahoo Finance
    """

    def __init__(self,
            pair,
            start_date,
            end_date,
            interval='hour') -> None:
        """Initialize a YahooDataProvider

        Args:
            pair (str): exemple BTCUSD
            start_date (datetime.datetime): First date to get. Format : YYYY-MM-DD
            end_date (datetime.datetime): Last date to get. Format : YYYY-MM-DD
            interval (str, optional): Can be day, hour, or minute.
        """
        super().__init__(pair, interval, start_date, end_date)
        self.pair = pair
        # add one interval to the end depending on the interval
        end_date = datetime.strptime(end_date, '%Y-%m-%d')
        if interval == 'day':
            self.end_date = end_date + timedelta(days=1)
        elif interval == 'hour':
            self.end_date = end_date + timedelta(hours=1)
        elif interval == 'minute':
            self.end_date = end_date + timedelta(minutes=1)
        else:
            raise ValueError('Interval must be day, hour or minute')

    def getData(self) -> pd.DataFrame :
        """Returns a pandas dataframe with the data.

        Returns:
            pd.DataFrame: _description_
        """        
        # convert interval into yahoo format
        yinterval = yahoointervalconverter[self.interval]
        # convert pair into yahoo format
        ypair = f'{self.pair[:3]}-{self.pair[3:]}'

        data = yf.Ticker(ypair)
        data = data.history(start=self.start_date,
                            end=self.end_date, interval=yinterval)
        try:
            del data["Dividends"]
        except:
            pass
        try:
            del data["Stock Splits"]
        except:
            pass
        try:
            del data['Time']
        except:
            pass
        data.rename(columns={
            'Open': 'open', 
            'High': 'high', 
            'Low': 'low', 
            'Close': 'close', 
            'Volume': 'volume'
            }, inplace=True)
        data['date'] = data.index
        data.index = data['date']
        data.drop(columns=['date'], inplace=True)
        data = self.normalizeColumnsOrder(data)
        self.checkDataframe(data)
        return data

class CSVDataProvider(DataProvider):
    """
    Get data from CSV file. The file name must be in the format {pair}-{interval}.csv
    """

    def __init__(self,
        pair,
        start_date,
        end_date,
        directory,
        interval='hour'):
        """Initialize a CSVDataProvider

        Args:
            pair (str): exemple BTCUSD
            start_date (datetime.datetime): First date to get. Format : YYYY-MM-DD.
            end_date (datetime.datetime): Last date to get. Format : YYYY-MM-DD.
            interval (str, optional): Can be day, hour, or minute.
        """
        super().__init__(pair, interval, start_date, end_date)
        self.directory = directory

    def getData(self) -> pd.DataFrame:
        """Returns a pandas dataframe with the data.
        """
        data = pd.read_csv(f'{self.directory}/f-{self.pair.lower()}-{self.interval}.csv')
        df =data.rename(columns={'Open': 'open', 
                                'High': 'high', 
                                'Low': 'low', 
                                'Close': 'close', 
                                'Volume': 'volume'})
        df.rename({'Unnamed: 0': 'date'}, axis=1, inplace=True)
        df.index = pd.to_datetime(df['date'])
        df.drop(columns=['date'], inplace=True)
        df = df[np.logical_and(df.index >= self.start_date, df.index <= self.end_date)]
        df = self.normalizeColumnsOrder(df)
        self.checkDataframe(df)
        return df


class ElasticDataProvider(DataProvider):
    """Get data from Elasticsearch. Index name must be in the format f-{pair}-{interval}.
       Main columns must be open, high, low, close, volume. And the date must be in the field @timestamp. 
    """
    def __init__(self,
            pair,
            start_date,
            end_date,
            es_url,
            es_user,
            es_pass,
            interval='hour') -> None:
        """Initialize a ElasticsearchDataprovider

        Args:
            pair (str): exemple BTCUSD
            start_date (datetime.datetime): First date to get. Format : YYYY-MM-DD.
            end_date (datetime.datetime): Last date to get. Format : YYYY-MM-DD.
            es_url (str): url of the elasticsearch server, example : https://localhost:9200
            es_user (str): name of the user for elasticsearch connection
            es_pass (str): password of the user for elasticsearch connection
            interval (str, optional): Can be day, hour, or minute.
        """
        super().__init__(pair, interval, start_date, end_date)
        self.es_url = es_url
        self.es_user = es_user
        self.es_pass = es_pass

    def connect(self):
        return Elasticsearch(
            self.es_url,
            http_compress=True,
            verify_certs=False,
            http_auth=(self.es_user, self.es_pass),
        )

    def download(self, from_, to, index_name):
        es = self.connect()
        query = {
            "query": {
                "bool" :{
                    "must" : {
                        "range": {
                            "@timestamp": {
                                "gte": from_,
                                "lte": to
                            }
                        }
                    }
                }
            }
        }
        result = es.search(index=index_name, body=query, size=10000)['hits']['hits']
        result = [x['_source'] for x in result]
        return result
    
    def download_data(self, pair, interval, from_, to):
        index_name = f'f-{pair.lower()}_{interval}'
        step = {
            'minute': timedelta(minutes=1),
            'hour' : timedelta(hours=1),
            'day' : timedelta(days=1),
            'week' : timedelta(weeks=1),
            'month' : timedelta(weeks=4),
        }
        beg = from_
        end = to
        end = to + step[interval] * 10000
        if end > to:
            end = to
        data = []
        while beg < to:

            data += self.download(beg, beg + step[interval] * 10000 if beg + step[interval] * 10000 < to else to, index_name)
            beg += step[interval] * 10000
            end += step[interval] * 10000
        data = pd.DataFrame(data)
        data.dropna(axis=1)
        data.rename({'@timestamp': 'date'}, axis=1, inplace=True)
        return data

    def getData(self):
        start = datetime.strptime(self.start_date, '%Y-%m-%d')
        end = datetime.strptime(self.end_date, '%Y-%m-%d')
        data = self.download_data(self.pair, self.interval, start, end)
        data.index = pd.to_datetime(data['date'])
        data.drop(columns=['date'], inplace=True)
        data = self.normalizeColumnsOrder(data)
        self.checkDataframe(data)
        return data


import sys
from datetime import datetime, timedelta
import pytz
import os

sys.path.insert(0, '/app')

from Hmile.DataProvider import YahooDataProvider
from Hmile.DataExporter import CSVDataExporter, ElasticDataExporter

PAIR = "BTCUSD"
START = "2021-12-01"
END = "2022-05-24"
INTERVAL = "hour"
ES_URL = "https://elastic:9200"
ES_USER = "elastic"
ES_PASS = "changeme"

PATH = '/output/csvdataexporter'

if __name__ == "__main__":
    os.mkdir('/output')
    os.mkdir(PATH)
    yf_dp = YahooDataProvider(PAIR, START, END, interval=INTERVAL)
    CSVDataExporter(yf_dp, PATH).export()
    ElasticDataExporter(yf_dp, ES_URL, ES_USER, ES_PASS).export()


import sys
import os
import argparse

import pandas as pd

from datetime import datetime

import sys
sys.path.insert(0, '/home/droid/8miles-render')
from Hmilerender.RabbitRenderer import RabbitRenderer
from random import randint


def fill_renderer(data, renderer, activate_date=True):
    for index, row in data.iterrows():
        # parse date from row["Date"] as YYYY-dd-mm
        if activate_date:
            date = datetime.strptime(row["Date"], "%Y-%m-%d")
        else:
            date = None
        renderer.append("open", row["open"], date)
        renderer.append("close", row["close"], date)
        renderer.append("high", row["high"], date)
        renderer.append("low", row["low"], date)
        renderer.append("volume", row["volume"], date)
        renderer.append("exit", row["exit"], date)
        renderer.append("long", row["long"], date)
        renderer.append("short", row["short"], date)
        renderer.append("money", row["money"], date)
        renderer.append("rew", randint(1,100), date)

if __name__ == "__main__":
    # parse args to get data_path and logs_dir
    parser = argparse.ArgumentParser()
    parser.add_argument("--data_path", help="path to data file", default='test/data/data.csv')
    parser.add_argument("--logs_dir", help="path to logs directory", default='logs')
    args = parser.parse_args()

    # read data
    data = pd.read_csv(args.data_path)

    # create renderer
    renderer = RabbitRenderer(
        args.logs_dir,
        activate_rew=True
    )
    # fill renderer with data
    fill_renderer(data, renderer)
    # render
    renderer.render()
    
    renderer.next_step()
    
    fill_renderer(data, renderer, activate_date=False)
    
    renderer.render()


from abc import abstractmethod

from elasticsearch import Elasticsearch, helpers

from Hmile.DataProvider import DataProvider

class DataExporter:
    """Export data to another format

    Args:
        DataProvider (Hmile.DataProvider.Dataprovider): Dataprovider to export
    """
    def __init__(self,
        dataprovider : DataProvider):
        self.dataprovider = dataprovider

    @abstractmethod
    def export(self):
        """Do export
        """
        raise NotImplementedError()

class CSVDataExporter(DataExporter):
    def __init__(self,
        dataprovider : DataProvider,
        directory : str):
        """Export data to csv. The file name will be in the format {pair}-{interval}.csv

        Args:
            dataprovider (Hmile.DataProvider.Dataprovider): Dataprovider to export
            directory (str): directory in with the csv will be saved
        """
        super().__init__(dataprovider)
        self.directory = directory

    def export(self):
        name = f'{self.directory}/f-{self.dataprovider.pair.lower()}-{self.dataprovider.interval}.csv'
        self.dataprovider.getData().to_csv(name, index=True)


class ElasticDataExporter(DataExporter):
    def __init__(
        self,
        dataprovider: DataProvider,
        es_url: str,
        es_user: str,
        es_pass: str):
        super().__init__(dataprovider)
        self.es_url = es_url
        self.es_user = es_user
        self.es_pass = es_pass
    
    def export(self):
        index_name = f'f-{self.dataprovider.pair.lower()}-{self.dataprovider.interval}'
        es = Elasticsearch(self.es_url, http_compress=True, verify_certs=False, http_auth=(self.es_user, self.es_pass))
        data = self.dataprovider.getData()
        helpers.bulk(es, ElasticDataExporter.doc_generator(data, index_name))


    def doc_generator(df, index_name):
        df_iter = df.iterrows()
        for index, document in df_iter:
            data_dict = document.to_dict()
            data_dict['@timestamp'] = index
            data_result = {}
            for k in data_dict:
                # if does not begin with obv
                if not k.startswith('obv'):
                    key_name = k.lower().replace('.', '_')
                    value = data_dict[k]
                    data_result[key_name] = value

            yield {
                    "_index": index_name,
                    'doc_type':'_doc',
                    "_id" : data_result['@timestamp'],
                    "_source": data_result,
            }


import sys
from datetime import datetime, timedelta
import pytz

sys.path.insert(0, '/app')

from Hmile.DataProvider import CSVDataProvider, ElasticDataProvider, YahooDataProvider
from Hmile.DataExporter import CSVDataExporter


interval_to_timedelta = {
    'minute': timedelta(minutes=1),
    'hour': timedelta(hours=1),
    'day': timedelta(days=1),
}

def test_dataprovider(dataprovider, start_date, end_date, interval):
    print(f'**** TESTING {dataprovider.__class__.__name__} ****')
    start = datetime.strptime(start_date, "%Y-%m-%d")
    end = datetime.strptime(end_date, "%Y-%m-%d")
    data = dataprovider.getData()
    print(data.head())
    assert data is not None
    first = data.index[0].to_pydatetime()
    first =  first.replace(tzinfo=pytz.utc)
    start = start.replace(tzinfo=pytz.utc)
    last = data.index[-1].to_pydatetime()
    last = last.replace(tzinfo=pytz.utc)
    end = end.replace(tzinfo=pytz.utc)
    second = data.index[1].to_pydatetime()
    second = second.replace(tzinfo=pytz.utc)
    assert first == start
    assert last == end
    assert second - first == interval_to_timedelta[interval]

PAIR = "BTCUSD"
START = "2022-01-01"
END = "2022-01-03"
INTERVAL = "hour"
ES_URL = "https://elastic:9200"
ES_USER = "elastic"
ES_PASS = "changeme"

if __name__ == "__main__":
    csv_dp = CSVDataProvider(PAIR, START, END, "test/data/csvdataprovider", interval=INTERVAL)
    yfinance_dp = YahooDataProvider(PAIR, START, END, interval=INTERVAL)
    elastic_dp = ElasticDataProvider(PAIR, START, END, ES_URL, ES_USER, ES_PASS, interval=INTERVAL)
    test_dataprovider(yfinance_dp, START, END, INTERVAL)
    test_dataprovider(csv_dp, START, END, INTERVAL)
    test_dataprovider(elastic_dp, START, END, INTERVAL)


import pandas as pd

class ColumnNameDoesNotExists(Exception):
    """Raised if user try to access to a not existing column in Hmilerender.Renderer.Renderer
    """
    def __init__(self, column_name) -> None:
        super().__init__(self)
        self.column_name = column_name
    
    def __str__(self) -> str:
        return """%s is not an existing data column""" % (self.column_name)
    

class DataframeFormatException(Exception):
    """Raise an exception if the dataframe does not correspond to the Hmile specification.
    """
    def __init__(self, msg : str, dataframe : pd.DataFrame) -> None:
        super().__init__(self)
        self.msg = msg
        self.dataframe = dataframe
    
    def __str__(self) -> str:
        return f'{self.msg} : {self.dataframe.head()}'

class DataProviderArgumentException(Exception):
    """Raise an exception if the data provider argument is not valid.
    """
    def __init__(self, msg : str) -> None:
        super().__init__(self)
        self.msg = msg
    
    def __str__(self) -> str:
        return f'{self.msg}'


import os

from .__version__ import __version__, __author__


RABBIT_BANNER =  """
   ______         .__.__          
 /  __  \  _____ |__|  |   ____  
 >      < /     \|  |  | _/ __ \ 
/   --   \  Y Y  \  |  |_\  ___/ 
\______  /__|_|  /__|____/\____>
       \/      \/              
**v{} by {}**
""" .format(__version__, __author__)


if not 'HMILE_BANNER' in os.environ or os.environ['HMILE_BANNER'] != 'disable':
    print(RABBIT_BANNER)


from distutils.core import setup
setup(
    name='hmile',
    version='0.2.1',
    author='theophanedroid',
    description='Python 3.x module to render financial results in tensorboard ',
    packages=['Hmile',],
    license='MIT license',
    long_description=open('README.md').read(),
    install_requires=[
        "mplfinance",
        "ta",
        "pandas",
        "elasticsearch",
        "torch",
        "numpy",
        "tensorflow",
        "tensorboard",
        "yfinance",
    ]
)


from datetime import datetime, timedelta
from threading import Thread
import pandas as pd
import copy

from .Exception import ColumnNameDoesNotExists

class RenderTask(Thread):
    """Render in a Thread
    """
    def __init__(self, func, render_params):
        """Create a render task

        Args:
            func (function): the function to start in thread
            render_params (dict): data to be passed as arguments of func
        """
        Thread.__init__(self)
        self.func = func
        self.render_params = render_params

    def run(self):
        self.func(self.render_params)

class Renderer:
    """
    Abstract class to guide the construction of a Renderer. To renderer multi-column, time series data.
    """
    def __init__(
        self,
        data_column_names : list,
        threaded=True
    ) -> None:
        """Create a renderer

        Args:
            data_column_names (list): the attributes name of time series data
            threaded (bool, optional): should the render method be started in a thread. Defaults to True.
        """
        self.columns_names = data_column_names
        self.threaded = threaded
        self.render_task = None
        self.init_columns()
        self.render_params = {}
        
    def init_columns(self):
        """Initialize column as empty pd.Series
        """
        self.columns = {
            col : pd.Series(dtype='float') for col in self.columns_names
        }
    
    def append(self,
               column_name : str,
               value : object,
               date : datetime = None) -> None:
        """Append a value to a column

        Args:
            column_name (str): column to select
            value (object): value to append
            date (datetime, optional): date for value. Defaults to None.

        Raises:
            ColumnNameDoesNotExists: raised if column_name does not exists in renderer
        """
        if date == None:
            date = datetime(2000, 8, 3) + timedelta(days=self.columns[column_name].shape[0])
        if column_name in self.columns :
            s = pd.Series([value], dtype='float', index=[date])
            self.columns[column_name] = pd.concat(
                [self.columns[column_name], s])
        else:
            raise ColumnNameDoesNotExists(column_name)
    
    def render_func(self, render_params : dict) -> None:
        """Implement here the rendering method. This method should not be called. Use render method

        Args:
            render_params (dict): parameters that should be passed to the thread

        Raises:
            NotImplemented: the function should be implemented
        """
        raise NotImplemented()
    
    def render(self):
        """Launch the rendering of all appended data
        """
        if self.threaded :
            if self.render_task:
                self.render_task.join(timeout=0.1)
            task = RenderTask(self.render_func, copy.deepcopy(self.render_params))
            self.render_task = task.start()
        else:
            self.render_func(copy.deepcopy(self.render_params))


import sys
from datetime import datetime, timedelta
import pytz
import os

sys.path.insert(0, '/app')

from Hmile.DataProvider import YahooDataProvider
from Hmile.DataTransformer import TaFeaturesTransformer

PAIR = "BTCUSD"
START = "2021-12-01"
END = "2022-05-24"
INTERVAL = "hou

PATH = '/output/csvdataexporter'

if __name__ == "__main__":
    yf_dp = YahooDataProvider(PAIR, START, END, interval=INTERVAL)
    data = TaFeaturesTransformer(yf_dp).transform()
    print(data.tail(10))


from datetime import datetime, timedelta
import mplfinance as mpf
import numpy as np
from matplotlib import pyplot as plt
from random import randint
import io
import IPython.display as IPydisplay
import tensorflow as tf
import threading
import numpy as np
import pandas as pd

from random import randint
from Hmilerender.Renderer import Renderer


additional_params = {
    'short' : {'marker' : 'v', 'color' :'g', 'panel' : 0, 'type' : 'scatter', 'markersize' : 100},
    'long' :  {'marker' : '^', 'color' :'b', 'panel' : 0, 'type' : 'scatter', 'markersize' : 100},
    'exit' :  {'marker' : 'x', 'color' :'r', 'panel' : 0, 'type' : 'scatter', 'markersize' : 100},
    'money' :  {'marker' : 'x', 'color' :'r', 'panel' : 3, 'type' : 'line', 'markersize' : 10, 'ylabel' : '\$\$ (red)'},
    'rew' :  {'color' :'b', 'panel' : 2, 'type' : 'line', 'ylabel' : 'rew (blue)'}
}


DATE_FORMAT = '%Y-%m-%d_%H:%M:%S'
# ajouter légende
# ajouter nom de la paire
class RabbitRenderer(Renderer):
    """Render in tensorboard
    """
    def __init__(self, 
                 logdir, 
                 name='Rabbit_render', 
                 tags=['rabbit'],
                 activate_volume=False,
                 activate_rew=False,
                 date_in_name=True):
        """Create a rabbitrenderer

        Args:
            logdir (str): directory to store tensorboard data
            name (str, optional): tensorboard data's name. Defaults to 'Rabbit_render'.
            tags (list, optional): tensorboard tags list. Defaults to ['rabbit'].
            activate_volume (bool, optional): should volume be shown. Defaults to False.
            activate_rew (bool, optional): should reward be shown. Defaults to False.
            date_in_name (bool, optional): should the name be shown in tensorboard data's name. Defaults to True.
        """
        super().__init__(
            ['open', 'close', 'low', 'high', 'volume', 'long', 'short', 'exit', 'money', 'rew'] ,
            threaded=False
        )
        self.logdir = logdir
        self.render_params['step'] = 0
        if date_in_name:
            self.name = name + '_' + datetime.now().strftime(DATE_FORMAT)
        else:
            self.name = name
        self.tags = tags
        self.activate_volume = activate_volume
    
    def render_func(self, render_params : dict):
        selected_cols = {k : v for k, v in self.columns.items() if k in ['open', 'close', 'low', 'high', 'volume']}
        ohlcv = pd.DataFrame(selected_cols)
        
        apd = []
        for key in additional_params.keys():
            col = self.columns[key]
            if not np.isnan(col).all():
                if 'panel' in additional_params[key] and additional_params[key]['panel'] > 1 and \
                      not self.activate_volume:
                    additional_params[key]['panel'] -=1
                a = mpf.make_addplot(col, **additional_params[key])
                apd.append(a)
        if not(self.logdir):
            mpf.plot(ohlcv, type='line', addplot=apd, volume=True, figsize=(10,7))
        else:
            buf = io.BytesIO()
            mpf.plot(ohlcv, type='line', addplot=apd, volume=self.activate_volume, figsize=(10,7), savefig=buf)
            image = tf.image.decode_image(buf.getvalue(), channels=4)
            image = tf.expand_dims(image, 0)
            writer = tf.summary.create_file_writer(self.logdir)
            with writer.as_default():
                summary_op = tf.summary.image(self.name, image, step=render_params['step'])
                writer.flush()
    
    def next_step(self):
        """Increment the tensorboard step data and reinitialize all column's data
        """
        self.render_params['step'] += 1
        self.init_columns()